@@ -24,10 +24,12 @@ _NO_DEFAULT = object()
 
 class Backend(object):
     lookup_table = {}
+    ovsdb_connection = None
 
-    def __init__(self, connection):
-        super(Backend, self).__init__()
-        self.start_connection(connection)
+    def __init__(self, connection, start=True, **kwargs):
+        super(Backend, self).__init__(**kwargs)
+        if start:
+            self.start_connection(connection)
 
     @classmethod
     def start_connection(cls, connection):
@@ -41,6 +43,11 @@ class Backend(object):
             LOG.exception(connection_exception)
             raise connection_exception
 
+    @classmethod
+    def restart_connection(cls):
+        cls.ovsdb_connection.stop()
+        cls.ovsdb_connection.start()
+
     @property
     def idl(self):
         return self.__class__.ovsdb_connection.idl
@@ -60,6 +67,9 @@ class Backend(object):
     def db_create(self, table, **col_values):
         return cmd.DbCreateCommand(self, table, **col_values)
 
+    def db_create_row(self, table, **col_values):
+        return cmd.DbCreateCommand(self, table, _as_row=True, **col_values)
+
     def db_destroy(self, table, record):
         return cmd.DbDestroyCommand(self, table, record)
 
@@ -94,7 +104,8 @@ class Backend(object):
 
     def lookup(self, table, record, default=_NO_DEFAULT):
         try:
-            return self._lookup(table, record)
+            with self.ovsdb_connection.lock:
+                return self._lookup(table, record)
         except idlutils.RowNotFound:
             if default is not _NO_DEFAULT:
                 return default
@@ -131,7 +142,9 @@ class Backend(object):
         if rl.table is None:
             raise idlutils.RowNotFound(table=table, col='uuid', match=record)
         if rl.column is None:
-            return next(iter(t.rows.values()))
+            if t.max_rows == 1:
+                return next(iter(t.rows.values()))
+            raise idlutils.RowNotFound(table=table, col='uuid', match=record)
         row = idlutils.row_by_value(self, rl.table, rl.column, record)
         if rl.uuid_column:
             rows = getattr(row, rl.uuid_column)
